sample_config = {
    'tableau_prod': {
        'server': 'https://<YOUR_SERVER>.com',
        'api_version': '<YOUR_API_VERSION>',
        'username': '<YOUR_USERNAME>',
        'password': '<YOUR_PASSWORD>',
        # comment out username / password and uncomment the lines below to use personal access token authentication
        # 'personal_access_token_name': '<YOUR_PERSONAL_ACCESS_TOKEN_NAME>',
        # 'personal_access_token_secret': '<YOUR_PERSONAL_ACCESS_TOKEN_SECRET>',
        'site_name': '<YOUR_SITE_NAME>',
        'site_url': '<YOUR_SITE_URL>'
    }
}
